@@ -6,7 +6,7 @@ from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 
 appName="Anki"
-appVersion="0.9.9.8.5"
+appVersion="1.0.0"
 appWebsite="http://ichi2.net/anki/download/"
 appWiki="http://ichi2.net/anki/wiki/"
 appHelpSite="http://ichi2.net/anki/wiki/AnkiWiki"
@@ -142,7 +142,7 @@ def run():
     import forms
     import ui
 
-    ui.splash = SplashScreen(5)
+    ui.splash = SplashScreen(3)
 
     import anki
     if anki.version != appVersion:
@@ -164,11 +164,6 @@ def run():
     conf = ankiqt.config.Config(
         unicode(os.path.abspath(opts.config), sys.getfilesystemencoding()))
 
-    # backups
-    from anki import DeckStorage
-    DeckStorage.backupDir = os.path.join(conf.configPath,
-                                         "backups")
-
     # qt translations
     translationPath = ''
     if 'linux' in sys.platform or 'unix' in sys.platform:
